﻿using SushiBarContracts.BindingModel;
using SushiBarContracts.SearchModel;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Models;
using SushiBarFileImplement.Models;

namespace SushiBarFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;

        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }

        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops.Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return _source.Shops.Where(x => x.ShopName.Contains(model.ShopName)).Select(x => x.GetViewModel).ToList();

        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Shops.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            _source.SaveShops();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var component = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            _source.SaveShops();
            return component.GetViewModel;
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                _source.Shops.Remove(element);
                _source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }

        public bool CheckCountSushi(ISushiModel model, int count)
        {
            int store = _source.Shops.Select(x => x.ShopSushis.Select(y =>
            (y.Value.Item1.Id == model.Id ? y.Value.Item2 : 0)).Sum()).Sum();
            return store >= count;
        }

        public bool SellSushis(ISushiModel model, int count)
        {
            var sushi = _source.Sushis.FirstOrDefault(x => x.Id == model.Id);
            if (sushi == null || !CheckCountSushi(model, count))
            {
                throw new ArgumentNullException("Такого количества суш нет, продать столько низя ");
            }

            foreach (var shop in _source.Shops)
            {
                var sushis = shop.ShopSushis;
                foreach (var elem in sushis.Where(x => x.Value.Item1.Id == sushi.Id))
                {
                    var selling = Math.Min(elem.Value.Item2, count);
                    sushis[elem.Value.Item1.Id] = (elem.Value.Item1, elem.Value.Item2 - selling);
                    count -= selling;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = model.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpening = shop.DateOpening,
                    ShopSushis = sushis,
                    MaxCountSushis = shop.MaxCountSushis
                });
            }

            _source.SaveShops();
            return true;
        }
    }
}
